/*
 * Copyright (c) Forge Development LLC and contributors
 * SPDX-License-Identifier: LGPL-2.1-only
 */

package com.minecraftserverzone.weaponmaster.gui.slider;

import java.text.DecimalFormat;

import org.lwjgl.input.Keyboard;

import com.minecraftserverzone.weaponmaster.gui.DetailedSettingsScreen;
import com.minecraftserverzone.weaponmaster.gui.UniqueItemSettingScreen;

import net.minecraft.client.Minecraft;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.TextComponentString;

/**
 * Slider widget implementation which allows inputting values in a certain range with optional step size.
 */
public class NewForgeSlider extends NewAbstractSlider
{
	public ITextComponent prefix;
    public ITextComponent suffix;

    public double minValue;
    public double maxValue;

    /** Allows input of discontinuous values with a certain step */
    public double stepSize;

    public boolean drawString;

    public final DecimalFormat format;

    /**
     * @param x x position of upper left corner
     * @param y y position of upper left corner
     * @param width Width of the widget
     * @param height Height of the widget
     * @param prefix {@link Component} displayed before the value string
     * @param suffix {@link Component} displayed after the value string
     * @param minValue Minimum (left) value of slider
     * @param maxValue Maximum (right) value of slider
     * @param currentValue Starting value when widget is first displayed
     * @param stepSize Size of step used. Precision will automatically be calculated based on this value if this value is not 0.
     * @param precision Only used when {@code stepSize} is 0. Limited to a maximum of 4 (inclusive).
     * @param drawString Should text be displayed on the widget
     */
    public NewForgeSlider(int x, int y, int width, int height, ITextComponent prefix, ITextComponent suffix, double minValue, double maxValue, double currentValue, double stepSize, int precision, boolean drawString)
    {
        super(x, y, width, height, new TextComponentString(""), 0D);
        this.prefix = prefix;
        this.suffix = suffix;
        this.minValue = minValue;
        this.maxValue = maxValue;
        this.stepSize = Math.abs(stepSize);
        this.value = this.snapToNearest((currentValue - minValue) / (maxValue - minValue));
        this.drawString = drawString;
        this.visible = false;

        if (stepSize == 0D)
        {
            precision = Math.min(precision, 4);

            StringBuilder builder = new StringBuilder("0");

            if (precision > 0)
                builder.append('.');

            while (precision-- > 0)
                builder.append('0');

            this.format = new DecimalFormat(builder.toString());
        }
        else if (this.equal(this.stepSize, Math.floor(this.stepSize)))
        {
            this.format = new DecimalFormat("0");
        }
        else
        {
            this.format = new DecimalFormat(Double.toString(this.stepSize).replaceAll("\\d", "0"));
        }

        this.updateMessage();
    }

    /**
     * Overload with {@code stepSize} set to 1, useful for sliders with whole number values.
     */
    public NewForgeSlider(int x, int y, int width, int height, ITextComponent prefix, ITextComponent suffix, double minValue, double maxValue, double currentValue, boolean drawString)
    {
        this(x, y, width, height, prefix, suffix, minValue, maxValue, currentValue, 1D, 0, drawString);
    }

    /**
     * @return Current slider value as a double
     */
    public double getValue()
    {
        return this.value * (maxValue - minValue) + minValue;
    }

    /**
     * @return Current slider value as an long
     */
    public long getValueLong()
    {
        return Math.round(this.getValue());
    }

    /**
     * @return Current slider value as an int
     */
    public int getValueInt()
    {
        return (int) this.getValueLong();
    }

    /**
     * @param value The new slider value
     */
    public void setValue(double value)
    {
        this.value = this.snapToNearest((value - this.minValue) / (this.maxValue - this.minValue));
        this.updateMessage();
    }

    public String getValueString()
    {
        return this.format.format(this.getValue());
    }

    @Override
    public void onClick(int mouseX, int mouseY)
    {
        super.onClick(mouseX, mouseY);
        this.setValueFromMouse(mouseX);
        setData();
    }

	@Override
    public void onDrag(Minecraft mc, int mouseX, int mouseY)
    {
        super.onDrag(mc, mouseX, mouseY);
        this.setValueFromMouse(mouseX); 
        setData();
    }
    
	public void setData() {
    	if(Minecraft.func_71410_x().field_71462_r.getClass() == DetailedSettingsScreen.class) {
    		DetailedSettingsScreen.setData();
        }else if(Minecraft.func_71410_x().field_71462_r.getClass() == UniqueItemSettingScreen.class) {
        	UniqueItemSettingScreen.setData();
        }
    }

    @Override
    public boolean keyPressed(int keyCode, int scanCode, int modifiers)
    {
        boolean flag = keyCode == Keyboard.KEY_LEFT;
        if (flag || keyCode == Keyboard.KEY_RIGHT)
        {
            if (this.minValue > this.maxValue)
                flag = !flag;
            float f = flag ? -1F : 1F;
            if (stepSize <= 0D)
                this.setSliderValue(this.value + (f / (this.width - 8)));
            else
                this.setValue(this.getValue() + f * this.stepSize);
        }

        return false;
    }

    public void setValueFromMouse(double mouseX)
    {
        this.setSliderValue((mouseX - (this.x + 4)) / (this.width - 8));
    }

    /**
     * @param value Percentage of slider range
     */
    public void setSliderValue(double value)
    {
        double oldValue = this.value;
        this.value = this.snapToNearest(value);
        if (!this.equal(oldValue, this.value))
            this.applyValue();

        this.updateMessage();
    }

    /**
     * Snaps the value, so that the displayed value is the nearest multiple of {@code stepSize}.
     * If {@code stepSize} is 0, no snapping occurs.
     */
    public double snapToNearest(double value)
    {
        if(stepSize <= 0D)
            return this.clamp(value, 0D, 1D);

        value = this.lerp(this.clamp(value, 0D, 1D), this.minValue, this.maxValue);

        value = (stepSize * Math.round(value / stepSize));

        if (this.minValue > this.maxValue)
        {
            value = this.clamp(value, this.maxValue, this.minValue);
        }
        else
        {
            value = this.clamp(value, this.minValue, this.maxValue);
        }

        return this.map(value, this.minValue, this.maxValue, 0D, 1D);
    }

    @Override
    public void updateMessage()
    {
        if (this.drawString)
        {
            this.setMessage(new TextComponentString("").func_150257_a(prefix).func_150258_a(this.getValueString()).func_150257_a(suffix));
        }
        else
        {
            this.setMessage(new TextComponentString(""));
        }
    }

    public void applyValue() {}

	public void setX(int p_265236_) {
		this.x = p_265236_;
	}

	public void setY(int p_265404_) {
		this.y = p_265404_;
	}

	public int getX() {
		return this.x;
	}

	public int getY() {
		return this.y;
	}
}
